/**
 * tsai.h
 * 
 *
 * @authors $Author: $ - Szemberg
 * @version $Id: $
 */



#ifndef _TSAI_H_
#define _TSAI_H_

/* Mtodo TSAI - Calibrao de Cameras - verso coplanar */

/************************************************************************************************/
/* Calibra a camera dados np pares de pontos.                                                   */
/*  Entrada:                                                                                    */
/*   np - nmero de pares de pontos - [ (xw,yw) , (xf, yf) ].                                   */
/*   (xw, yw) - ponto do mundo.                                                                 */
/*   (xf, yf) - ponto em coodernadas da tela em pixel (origem no centro da imagem e eixo ox da  */
/*                                                     esquerda para direita e eixo oy de baixo */
/*                                                     para cima).                              */
/*  Sada:                                                                                      */
/*   f - distncia focal.                                                                       */
/*   T - translao (origem do mundo em sistema de coordenadas da cmera).                      */
/*   r - rotao.                                                                               */
/************************************************************************************************/
int tsaiCalibration2D(int np, double *xw, double *yw, double *xf, double *yf,
                      double *f, double T[3], double r[9]);

/************************************************************************************************/
/* Calibra a camera dados np pares de pontos, exceto distncia focal.                           */
/*  Entrada:                                                                                    */
/*   np - nmero de pares de pontos - [ (xw,yw) , (xf, yf) ].                                   */
/*   (xw, yw) - ponto do mundo.                                                                 */
/*   (xf, yf) - ponto em coodernadas da tela em pixel (origem no centro da imagem e eixo ox da  */
/*                                                     esquerda para direita e eixo oy de baixo */
/*                                                     para cima).                              */
/*   f - distncia focal.                                                                       */
/*  Sada:                                                                                      */
/*   T - translao (origem do mundo em sistema de coordenadas da cmera).                      */
/*   r - rotao.                                                                               */
/************************************************************************************************/
int tsaiCalibration2D_Tr(int np, double *xw, double *yw, double *xf, double *yf, double f,
                         double T[3], double r[9]);

/************************************************************************************************/
/* Projeta o ponto (xw, yw, zw) na tela.                                                        */
/*  Entrada:                                                                                    */
/*   (xw, yw, zw) - ponto a ser projetado.                                                      */
/*   f - distncia focal.                                                                       */
/*   T - translao (origem do mundo em sistema de coordenadas da cmera).                      */
/*   r - rotao.                                                                               */
/*  Sada:                                                                                      */
/*   (xf, yf) - ponto projetado.                                                                */
/************************************************************************************************/
void tsaiProject(double xw, double yw, double zw, double *xf, double *yf,
                 double f, double T[3], double r[9]);

/************************************************************************************************/
/* Inverte a projeo do ponto da tela.                                                         */
/*  Entrada:                                                                                    */
/*   (xf, yf) - ponto projetado da tela.                                                        */
/*   f - distncia focal.                                                                       */
/*   T - translao (origem do mundo em sistema de coordenadas da cmera).                      */
/*   r - rotao.                                                                               */
/*   zw - coordenada z do ponto do mundo.                                                       */
/*  Sada:                                                                                      */
/*   (xw, yw, zw) - ponto do mundo. Note que zw deve ser dado de entrada.                       */
/************************************************************************************************/
void tsaiUnproject(double xf, double yf, double *xw, double *yw, double zw,
                   double f, double T[3], double r[9]);

/************************************************************************************************/
/* Encontra a posio da cmera no sistema de coordenadas do mundo                              */
/*  Entrada:                                                                                    */
/*   T - translao (origem do mundo em sistema de coordenadas da cmera).                      */
/*   r - rotao.                                                                               */
/*  Sada:                                                                                      */
/*   (x, y, z) - coordenadas da posio da cmera.                                              */
/************************************************************************************************/
void tsaiCameraPosition(double *x, double *y, double *z,
                        double T[3], double r[9]);

/************************************************************************************************/
/* Retorna a matriz Model View a ser utilizado no OpenGL.                                       */
/*  Entrada:                                                                                    */
/*   T - translao (origem do mundo em sistema de coordenadas da cmera).                      */
/*   r - rotao.                                                                               */
/*  Sada:                                                                                      */
/*   mv - matriz Model View.                                                                    */
/************************************************************************************************/
void tsaiOGLModelViewMatrix(double T[3], double r[9], double mv[16]);

/************************************************************************************************/
/* Retorna a matriz de projeo a ser utilizado no OpenGL.                                      */
/*  Entrada:                                                                                    */
/*   f - distncia focal.                                                                       */
/*   nearplane - plano near do frustum de viso.                                                */
/*   farplane - plano far do frustum de viso.                                                  */
/*   width - largura do frustum de viso.                                                       */
/*   height - altura do frustum de viso.                                                       */
/*  Sada:                                                                                      */
/*   pr - matriz de projeo.                                                                   */
/************************************************************************************************/
void tsaiOGLProjectionMatrix(double f, double nearplane, double farplane, int width, int height, double pr[16]);

/************************************************************************************************/
/* Reamostra os pares de pontos segundo pares de pontos originais.                              */
/*  Entrada:                                                                                    */
/*   np - nmero de pares de pontos - [ (xw,yw) , (xf, yf) ].                                   */
/*   (xw, yw) - ponto do mundo.                                                                 */
/*   (xf, yf) - ponto em coodernadas da tela em pixel (origem no centro da imagem e eixo ox da  */
/*                                                     esquerda para direita e eixo oy de baixo */
/*                                                     para cima).                              */
/*   npnx - nmero de pares de pontos novos na direo x do modelo. (Max. 10)                   */
/*   npny - nmero de pares de pontos novos na direo y do modelo. (Max. 10)                   */
/*  Sada:                                                                                      */
/*   (xwn, ywn) - ponto novo do mundo.                                                          */
/*   (xfn, yfn) - ponto novo em coodernadas da tela em pixel.                                   */
/************************************************************************************************/
int tsaiBuildNewData(int np, double *xw, double *yw, double *xf, double *yf,
                     int npnx, int npny, double *xwn, double *ywn, double *xfn, double *yfn);

#endif